﻿using System.Net.Sockets;
using System.Net;
using System;
using System.Threading.Tasks;

namespace EonaCat.Network
{
    public class SocketTcpServer
    {
        public event Action<RemoteInfo> OnConnect;
        public event Action<RemoteInfo> OnReceive;
        public event Action<RemoteInfo> OnSend;
        public event Action<RemoteInfo> OnDisconnect;

        private TcpListener listener;

        public SocketTcpServer(IPAddress ipAddress, int port)
        {
            listener = new TcpListener(ipAddress, port);
        }

        public SocketTcpServer(string ipAddress, int port)
        {
            IPAddress address = IPAddress.Parse(ipAddress);
            listener = new TcpListener(address, port);
        }

        public async Task Start()
        {
            listener.Start();
            while (true)
            {
                try
                {
                    TcpClient client = await listener.AcceptTcpClientAsync().ConfigureAwait(false);
                    Socket socket = client.Client;
                    OnConnect?.Invoke(new RemoteInfo { Socket = socket, IsTcp = true, IsIpv6 = socket.RemoteEndPoint.AddressFamily == AddressFamily.InterNetworkV6, EndPoint = socket.RemoteEndPoint });
                    _ = StartReceiving(socket);
                }
                catch (SocketException ex)
                {
                    Console.WriteLine($"SocketException: {ex.Message}");
                }
            }
        }

        private async Task StartReceiving(Socket socket)
        {
            byte[] buffer = new byte[1024];
            while (socket.Connected)
            {
                try
                {
                    int received = await socket.ReceiveAsync(new ArraySegment<byte>(buffer), SocketFlags.None).ConfigureAwait(false);
                    if (received > 0)
                    {
                        byte[] data = new byte[received];
                        Array.Copy(buffer, data, received);
                        OnReceive?.Invoke(new RemoteInfo { Socket = socket, IsTcp = true, IsIpv6 = socket.RemoteEndPoint.AddressFamily == AddressFamily.InterNetworkV6, EndPoint = socket.RemoteEndPoint, Data = data});
                    }
                    else
                    {
                        break;
                    }
                }
                catch (SocketException ex)
                {
                    Console.WriteLine($"SocketException: {ex.Message}");
                    break;
                }
            }
            OnDisconnect?.Invoke(new RemoteInfo {Socket = socket, IsIpv6 = socket.RemoteEndPoint.AddressFamily == AddressFamily.InterNetworkV6, IsTcp = true, EndPoint = socket.RemoteEndPoint });
        }

        public async Task SendTo(Socket socket, byte[] data)
        {
            await socket.SendAsync(new ArraySegment<byte>(data), SocketFlags.None).ConfigureAwait(false);
            OnSend?.Invoke(new RemoteInfo { Socket = socket, IsTcp = true, EndPoint = socket.RemoteEndPoint, IsIpv6 = socket.RemoteEndPoint.AddressFamily == AddressFamily.InterNetworkV6, Data = data });
        }
    }
}